#!/usr/bin/env python
#Tiago de Freitas Pereira <tiagofrepereira@gmail.com>
# Sun Oct 21 20:51:19 CET 2012

"""
This script will run the result analisys for the DoG countermeasure


"""

import os, sys
import argparse
import bob
import numpy

from .. import spoof

from antispoofing.utils.ml import *
from antispoofing.utils.helpers import *
from antispoofing.utils.db import *

def main():

  ##########
  # General configuration
  ##########


  parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

  parser.add_argument('-s', '--scores-dir', type=str, dest='scoresDir', default='', help='Base directory containing the Scores to a specific protocol  (defaults to "%(default)s")')

  parser.add_argument('-n','--score-normalization',type=str, dest='scoreNormalization',default='', choices=('znorm','minmax',''))

  parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False, help='Increases this script verbosity')

  #######
  # Database especific configuration
  #######
  Database.create_parser(parser, implements_any_of='video')

  args = parser.parse_args()
  lines  = ['r']
  energy = 0.0
  scoresRange = (-5,5)

  ## Parsing
  scoresDir          = args.scoresDir
  scoreNormalization = args.scoreNormalization
  verbose            = args.verbose

  if not os.path.exists(scoresDir):
    parser.error("scores-dir directory does not exist")

  #########
  # Loading some dataset
  #########
  if(verbose):
    print("Querying the database ... ")

  database = args.cls(args)

  trainReal, trainAttack = database.get_train_data()
  develReal, develAttack = database.get_devel_data()
  testReal, testAttack   = database.get_test_data()


  #### Storing the scores in order to plot their distribution
  trainRealScores   = []
  trainAttackScores = []

  develRealScores   = []
  develAttackScores = []

  testRealScores    = []
  testAttackScores  = []

  thresholds        = []
  develTexts        = []
  testTexts        = []

  if(verbose):
    print("Generating test results ....")



  #Getting the scores
  realScores   = ScoreReader(trainReal,scoresDir)
  attackScores = ScoreReader(trainAttack,scoresDir)
  train_real_scores = realScores.getScores()
  train_attack_scores = attackScores.getScores()

  realScores   = ScoreReader(develReal,scoresDir)
  attackScores = ScoreReader(develAttack,scoresDir)
  devel_real_scores = realScores.getScores()
  devel_attack_scores = attackScores.getScores()

  realScores   = ScoreReader(testReal,scoresDir)
  attackScores = ScoreReader(testAttack,scoresDir)
  test_real_scores = realScores.getScores()
  test_attack_scores = attackScores.getScores()
 
  scoreNorm = ScoreNormalization(numpy.concatenate((train_real_scores,train_attack_scores)))

  #Applying the score normaliztion
  if(scoreNormalization=="minmax"):

    train_real_scores   = scoreNorm.calculateMinMaxNorm(train_real_scores)
    train_attack_scores = scoreNorm.calculateMinMaxNorm(train_attack_scores)

    devel_real_scores   = scoreNorm.calculateMinMaxNorm(devel_real_scores)
    devel_attack_scores = scoreNorm.calculateMinMaxNorm(devel_attack_scores)

    test_real_scores   = scoreNorm.calculateMinMaxNorm(test_real_scores)
    test_attack_scores = scoreNorm.calculateMinMaxNorm(test_attack_scores)

    scoresRange = (-1,1)
  
  elif(scoreNormalization=="znorm"):

    train_real_scores   = scoreNorm.calculateZNorm(train_real_scores)
    train_attack_scores = scoreNorm.calculateZNorm(train_attack_scores)

    devel_real_scores   = scoreNorm.calculateZNorm(devel_real_scores)
    devel_attack_scores = scoreNorm.calculateZNorm(devel_attack_scores)

    test_real_scores   = scoreNorm.calculateZNorm(test_real_scores)
    test_attack_scores = scoreNorm.calculateZNorm(test_attack_scores)
 

  if numpy.mean(devel_real_scores) < numpy.mean(devel_attack_scores):
    train_real_scores = train_real_scores * -1; train_attack_scores = train_attack_scores * -1
    devel_real_scores = devel_real_scores * -1; devel_attack_scores = devel_attack_scores * -1
    test_real_scores = test_real_scores * -1; test_attack_scores = test_attack_scores * -1

    
   #### Calculation of the error rates for one plane
  (test_hter,devel_hter),(test_text,devel_text),thres = perf.perf_hter([test_real_scores,test_attack_scores], [devel_real_scores,devel_attack_scores], bob.measure.eer_threshold)
    
  print(devel_text)
  print(test_text)
 
  return 0


if __name__ == "__main__":
  main()
